(* Lash *)
(* ported from Satallax file: atom.ml *)

open Syntax
open Log

type atom = int
type literal = int

let next_atom = ref 1
let atom_hash = Ftm.hashtbli1_make 1024
let atom_hash_rev = Ftm.vectori_make 1000

let new_atom_actions = ref []

let max_atom () = (!next_atom) - 1


let new_atom m =
  let a = (!next_atom) in
  incr next_atom;
  Ftm.hashtbli1_add atom_hash m a;
  Ftm.vectori_add atom_hash_rev a m;
  if (!verbosity > 4) then
    if (!verbosity > 8) then Printf.printf "Atom %d: %s\n" a (ftm_str m)
    else (Printf.printf "Atom %d\n" a);
  List.iter (fun f -> f m a) !new_atom_actions;
  a

let get_atom m =
  let ret = Ftm.hashtbli1_find atom_hash m in
  if ret = 0 then new_atom m else ret

let assignedp m =
  match fneg_body m with
    Some n -> Ftm.hashtbli1_mem atom_hash n
  | None -> Ftm.hashtbli1_mem atom_hash m

let atom_to_trm = Ftm.vectori_find atom_hash_rev

let rec get_literal m =
  match fneg_body m with
    Some n -> - (get_literal n)
  | None -> get_atom m

let literal_to_trm l =
  if (l < 0)
  then fneg (Ftm.vectori_find atom_hash_rev (- l))
  else Ftm.vectori_find atom_hash_rev l


let print_minisat eval b i m =
  let pol v = if v = 0 then "+" else "-"
  and v = eval i in
  match b with
    (*** 0 means true ***)
    true  -> if (v = 0) then Printf.printf "%d %s\n" i (ftm_str m)
  | false -> if (v < 2) then Printf.printf "%d%s %s\n" i (pol v) (ftm_str m)

let print_model eval b =
  for i = 1 to !next_atom -1 do
    print_minisat eval b i (Ftm.vectori_find atom_hash_rev i)
  done
